import json
import logging
import os
import time
from collections import defaultdict
from pathlib import Path

import typer
from rich.logging import RichHandler

from .cost import calculate_cost
from .doc_parser import parse_doc
from .docling_translator import translate_doc, translate_doc_parallel
from .docling_wrapper import WrapDoclingDocument
from .highlighter import get_highlight_colors
from .translator import StatefulTranslator

logging.basicConfig(
    level=logging.DEBUG, handlers=[RichHandler()], format="%(message)s", datefmt="[%X]"
)

logger = logging.getLogger(__name__)
for name in ["openai", "httpcore", "PIL", "Comm"]:
    logging.getLogger(name).setLevel(logging.INFO)


def cli(
    input_doc_path: str = typer.Option(..., help="Input document path"),
    output_dir: str = typer.Option(..., help="Output directory"),
    translation_model: str = typer.Option(..., help="Translation model"),
    score_model: str = typer.Option(..., help="Score model"),
    ignore_labels: str = typer.Option(..., help="Ignore labels"),
    max_retries: int = typer.Option(3, help="Max retries"),
    retry_delay: float = typer.Option(1.0, help="Retry delay"),
    parallel: int = typer.Option(10, help="Parallel translation"),
):
    input_doc_path = Path(input_doc_path)
    output_dir = Path(output_dir)

    start = time.time()
    ignore_labels = set(ignore_labels.split(","))
    stateful_translator = StatefulTranslator.load(
        path=Path(output_dir) / "stateful_translator.json",
        translation_model=translation_model,
        score_model=score_model,
    )

    os.makedirs(output_dir, exist_ok=True)
    logger.info(f"===Parsing {input_doc_path}===")
    conv_doc = parse_doc(input_doc_path, output_dir, force_reparse=False)
    wrap_doc = WrapDoclingDocument.from_docling_doc(conv_doc)

    logger.info(f"===Translating {input_doc_path}===")
    if stateful_translator is None:
        stateful_translator = StatefulTranslator.load(
            translation_model=translation_model, score_model=score_model
        )
    try:
        if parallel > 1:
            translated_doc = translate_doc_parallel(
                wrap_doc, stateful_translator, max_retries, retry_delay, parallel
            )
        else:
            translated_doc = translate_doc(
                wrap_doc, stateful_translator, max_retries, retry_delay
            )
    except Exception as e:
        logger.error(f"Error translating {input_doc_path}: {e}")
        raise e
    except KeyboardInterrupt:
        logger.error(f"Keyboard interrupt")
        stateful_translator.save(output_dir / "stateful_translator.json")
        exit(1)
    stateful_translator.save(output_dir / "stateful_translator.json")

    translated_json_path = output_dir / f"document.translated.json"
    logger.info(f"===Saving {translated_json_path}===")
    with open(translated_json_path, "w") as f:
        f.write(translated_doc.model_dump_json())

    translated_md_path = output_dir / f"document.translated.md"
    logger.info(f"===Saving {translated_md_path}===")
    with open(translated_md_path, "w") as f:
        f.write(translated_doc.export_to_markdown())

    def color_doc(threshold: int):
        # 输出染色后的docx
        ref_to_highlights = get_highlight_colors(stateful_translator.cache, threshold)

        # 忽略指定标签的docx
        translated_docx_path = (
            output_dir
            / f"document.translated.{threshold}-{'-'.join(ignore_labels)}.docx"
        )
        logger.info(f"===Saving with ignore {translated_docx_path}===")
        docx_doc = translated_doc.export_to_docx(
            ref_to_highlights=ref_to_highlights,
            ignore_labels=ignore_labels,
        )
        docx_doc.save(translated_docx_path)

        # 输出完整的染色后的docx
        logger.info(f"===Saving {translated_docx_path}===")
        translated_full_docx_path = output_dir / f"document.translated.{threshold}.docx"
        full_docx_doc = translated_doc.export_to_docx(
            ref_to_highlights=ref_to_highlights,
        )
        full_docx_doc.save(translated_full_docx_path)

        highlight_to_word_count = defaultdict(int)
        for k, v in ref_to_highlights.items():
            highlight_to_word_count[v] += len(stateful_translator.cache[k]["raw_text"])

        logger.info(f"Color statistics: {highlight_to_word_count}")

    for threshold in [17, 18, 19, 20]:
        color_doc(threshold)

    end = time.time()
    logger.info(f"Time taken: {end - start:.2f} seconds")

    simple_summary = {
        "translation_elapsed": stateful_translator.translation_elapsed,
        "score_elapsed": stateful_translator.score_elapsed,
        "total_words": stateful_translator.total_words,
        "translation_usage": stateful_translator.translation_usage,
        "translation_cost": calculate_cost(
            stateful_translator.translation_usage, translation_model
        ),
        "score_usage": stateful_translator.score_usage,
        "score_cost": calculate_cost(stateful_translator.score_usage, score_model),
        "total_revise_count": stateful_translator.total_revise_count,
    }

    for k, v in simple_summary.items():
        logger.info(f"{k}: {v}")

    with open(output_dir / "summary.json", "w") as f:
        json.dump(simple_summary, f, indent=4)


if __name__ == "__main__":
    typer.run(cli)
