import logging
from pathlib import Path

from docling.datamodel.base_models import InputFormat
from docling.datamodel.document import DoclingDocument
from docling.datamodel.pipeline_options import PdfPipelineOptions
from docling.document_converter import (
    ConversionResult,
    DocumentConverter,
    PdfFormatOption,
)

logger = logging.getLogger(__name__)


def parse_doc(
    input_path: Path,
    output_dir: Path,
    force_reparse: bool = False,
    image_scale: float = 2.0,
) -> DoclingDocument:
    output_doc_path = output_dir / "document.json"
    output_markdown_path = output_dir / "document.md"
    if not force_reparse and output_doc_path.exists():
        logger.info(f"Loading cached document from {output_doc_path}")
        with open(output_doc_path, "r") as f:
            return DoclingDocument.model_validate_json(f.read())

    # Important: For operating with page images, we must keep them, otherwise the DocumentConverter
    # will destroy them for cleaning up memory.
    # This is done by setting PdfPipelineOptions.images_scale, which also defines the scale of images.
    # scale=1 correspond of a standard 72 DPI image
    # The PdfPipelineOptions.generate_* are the selectors for the document elements which will be enriched
    # with the image field
    pipeline_options = PdfPipelineOptions()
    pipeline_options.images_scale = image_scale
    pipeline_options.generate_page_images = True
    pipeline_options.generate_picture_images = True
    pipeline_options.artifacts_path = ".models"

    doc_converter = DocumentConverter(
        format_options={
            InputFormat.PDF: PdfFormatOption(pipeline_options=pipeline_options)
        }
    )
    logger.info(f"Parsing {input_path}")

    conv_res: ConversionResult = doc_converter.convert(input_path)
    conv_doc: DoclingDocument = conv_res.document

    logger.info(f"Saving document to {output_doc_path}")
    with open(output_doc_path, "w") as f:
        f.write(conv_doc.model_dump_json())

    with open(output_markdown_path, "w") as f:
        f.write(conv_doc.export_to_markdown())

    return conv_doc
