import logging
import re

from docling_core.types.doc import ImageRefMode, PictureItem, TableItem, TextItem
from rich.progress import (
    BarColumn,
    Progress,
    ProgressColumn,
    Task,
    TaskProgressColumn,
    TextColumn,
    TimeElapsedColumn,
    TimeRemainingColumn,
)
from rich.text import Text

from .docling_wrapper import WrapDoclingDocument
from .translator import StatefulTranslator

logger = logging.getLogger(__name__)


class GoodBadColumn(ProgressColumn):
    def render(self, task: "Task") -> Text:
        total = task.total
        if task.fields.get("good") is None or task.fields.get("bad") is None:
            return Text(f"{total}-0-0", style="progress.filesize")
        return Text(
            f"{total}-{task.fields['good']}-{task.fields['bad']}",
            style="progress.filesize",
        )


def translate_text_element(
    context, element: TextItem, stateful_translator: StatefulTranslator
) -> TextItem:
    if not element.text.strip():
        return element
    # 没有中文不翻译
    if not re.search(r"[\u4e00-\u9fa5]", element.text):
        return element
    element.orig = element.text
    element.text = stateful_translator.translate(
        element.self_ref, element.text, element.label
    )
    return element


def translate_table_element(
    context, element: TableItem, stateful_translator: StatefulTranslator
) -> TableItem:
    # TODO: translate table header
    return element


def translate_element(element, context, stateful_translator, max_retries, retry_delay):
    retries = 0
    while retries < max_retries:
        try:
            if isinstance(element, TextItem):
                element = translate_text_element(context, element, stateful_translator)
            elif isinstance(element, TableItem):
                element = translate_table_element(context, element, stateful_translator)
            elif isinstance(element, PictureItem):
                # skip
                pass
            else:
                logger.warning(f"Unknown element type: {type(element)}")
            return element, True, retries
        except Exception as e:
            import traceback

            traceback.print_exc()
            retries += 1
            time.sleep(retry_delay)
    return element, False, retries


def translate_doc(
    conv_doc: WrapDoclingDocument,
    stateful_translator: StatefulTranslator,
    max_retries: int = 3,
    retry_delay: float = 1.0,
):
    context = {}
    total_items = 0
    bad_items = 0
    good_items = 0
    for element, _level in conv_doc.iterate_items():
        total_items += 1

    with Progress(
        TextColumn("[progress.description]{task.description}"),
        TimeElapsedColumn(),
        BarColumn(),
        TaskProgressColumn(),
        TimeRemainingColumn(),
        GoodBadColumn(),
        speed_estimate_period=300,
    ) as progress_bar:
        task = progress_bar.add_task(description="Translating", total=total_items)

        for element, _level in conv_doc.iterate_items():
            element, success, retries = translate_element(
                element, context, stateful_translator, max_retries, retry_delay
            )
            if success:
                good_items += 1
            else:
                bad_items += 1
            progress_bar.update(task, good=good_items, bad=bad_items, advance=1)

    return conv_doc


import time
from concurrent.futures import ThreadPoolExecutor, as_completed


def translate_doc_parallel(
    conv_doc, stateful_translator, max_retries=3, retry_delay=1.0, parallel_workers=2
):
    context = {}
    total_items = 0
    bad_items = 0
    good_items = 0
    for element, _level in conv_doc.iterate_items():
        total_items += 1

    with Progress(
        TextColumn("[progress.description]{task.description}"),
        TimeElapsedColumn(),
        BarColumn(),
        TaskProgressColumn(),
        TimeRemainingColumn(),
        GoodBadColumn(),
    ) as progress_bar:
        task = progress_bar.add_task(description="Translating", total=total_items)

        with ThreadPoolExecutor(max_workers=parallel_workers) as executor:
            futures = [
                executor.submit(
                    translate_element,
                    element,
                    context,
                    stateful_translator,
                    max_retries,
                    retry_delay,
                )
                for element, _level in conv_doc.iterate_items()
            ]
            for future in as_completed(futures):
                element, success, retries = future.result()
                if success:
                    good_items += 1
                else:
                    bad_items += 1
                progress_bar.update(task, good=good_items, bad=bad_items, advance=1)

    return conv_doc
