import io
import logging
import sys
from typing import Optional, Set

from docling.datamodel.document import DoclingDocument
from docling.datamodel.pipeline_options import PdfPipelineOptions
from docling.document_converter import (
    ConversionResult,
    DocumentConverter,
    PdfFormatOption,
)
from docling_core.types.doc.document import (
    DEFAULT_EXPORT_LABELS,
    CodeItem,
    DocItem,
    DocItemLabel,
    ImageRefMode,
    ListItem,
    PictureItem,
    SectionHeaderItem,
    TableItem,
    TextItem,
)
from docx import Document
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
from docx.shared import Inches, Pt, RGBColor
from rich.progress import track

logger = logging.getLogger(__name__)


class WrapDoclingDocument(DoclingDocument):
    @classmethod
    def from_docling_doc(cls, doc: DoclingDocument):
        return cls(**doc.model_dump())

    def export_to_docx(
        self,
        ref_to_highlights: dict = {},
        ignore_labels: Set[DocItemLabel] = set(),
        from_element: int = 0,
        to_element: int = sys.maxsize,
        labels: Set[DocItemLabel] = DEFAULT_EXPORT_LABELS,
        strict_text: bool = False,
        image_placeholder: str = "<!-- image -->",
        image_mode: ImageRefMode = ImageRefMode.EMBEDDED,
        page_no: Optional[int] = None,
    ) -> Document:
        """
        Serialize to a .docx document.

        :param from_element: Body slicing start index (inclusive).
        :param to_element: Body slicing stop index (exclusive).
        :param labels: The set of document labels to include in the export.
        :param strict_text: Whether to only include the text content of the document.
        :param image_placeholder: Placeholder to use for images in the document.
        :param image_mode: Mode for including images.
        :param page_no: Optional page number for filtering items.
        :returns: A docx Document object.
        """
        doc = Document()

        list_nesting_level = 0
        previous_level = 0
        in_list = False

        count = 0
        for ix, (item, level) in enumerate(
            self.iterate_items(self.body, with_groups=True, page_no=page_no)
        ):
            count += 1

        for ix, (item, level) in track(
            enumerate(self.iterate_items(self.body, with_groups=True, page_no=page_no)),
            description="Exporting to docx",
            total=count,
        ):
            if level < previous_level:
                level_difference = previous_level - level
                list_nesting_level = max(0, list_nesting_level - level_difference)

            previous_level = level

            if ix < from_element or to_element <= ix:
                continue

            if isinstance(item, DocItem) and item.label not in labels:
                continue

            if item.label in ignore_labels:
                continue

            if isinstance(item, TextItem) and item.label in [DocItemLabel.TITLE]:
                paragraph = doc.add_heading(level=0)
                run = paragraph.add_run(item.text.strip())
                if item.self_ref in ref_to_highlights:
                    run.font.color.rgb = ref_to_highlights[item.self_ref]

            elif (
                isinstance(item, TextItem)
                and item.label in [DocItemLabel.SECTION_HEADER]
            ) or isinstance(item, SectionHeaderItem):
                heading_level = min(level, 4)
                paragraph = doc.add_heading(level=heading_level)
                run = paragraph.add_run(item.text.strip())
                if item.self_ref in ref_to_highlights:
                    run.font.color.rgb = ref_to_highlights[item.self_ref]

            elif isinstance(item, CodeItem) and item.label in labels:
                paragraph = doc.add_paragraph()
                run = paragraph.add_run(item.text)
                run.font.name = "Courier New"
                run.font.size = Pt(10)
                # paragraph.style = "Code"

            elif isinstance(item, ListItem) and item.label in [DocItemLabel.LIST_ITEM]:
                paragraph = doc.add_paragraph()
                if item.enumerated:
                    paragraph.add_run(f"{item.marker} ").bold = True
                else:
                    paragraph.add_run("- ").bold = True
                run = paragraph.add_run(item.text)
                if item.self_ref in ref_to_highlights:
                    run.font.color.rgb = ref_to_highlights[item.self_ref]

            elif isinstance(item, TextItem) and item.label in labels:
                paragraph = doc.add_paragraph()
                run = paragraph.add_run(item.text.strip())
                if item.self_ref in ref_to_highlights:
                    run.font.color.rgb = ref_to_highlights[item.self_ref]

            elif isinstance(item, TableItem) and not strict_text:
                WrapTableItem.from_docling_table(item).export_to_docx(doc)

            elif isinstance(item, PictureItem) and not strict_text:
                if image_mode == ImageRefMode.EMBEDDED:
                    paragraph = doc.add_paragraph()
                    buffer = io.BytesIO()
                    image = item.get_image(self)
                    image.save(buffer, format="PNG")
                    doc.add_picture(buffer, width=Inches(6))
                elif image_mode == ImageRefMode.PLACEHOLDER:
                    paragraph = doc.add_paragraph()
                    paragraph.text = image_placeholder
                else:
                    raise ValueError(f"Unknown image mode: {image_mode}")

            elif isinstance(item, DocItem) and item.label in labels:
                paragraph = doc.add_paragraph()
                paragraph.text = "<missing-text>"

        return doc


class WrapTableItem(TableItem):
    @classmethod
    def from_docling_table(cls, table: TableItem):
        return cls(**table.model_dump())

    def export_to_docx(self, doc: Document):
        table = doc.add_table(rows=len(self.data.grid), cols=len(self.data.grid[0]))
        for row_idx, row in enumerate(self.data.grid):
            row_cells = table.rows[row_idx].cells
            for col_idx, col in enumerate(row):
                row_cells[col_idx].text = col.text
        return table
