import os
import typing

import dotenv
import openai
from openai import OpenAI
from pydantic import BaseModel, Field
from pydantic_settings import BaseSettings, SettingsConfigDict

dotenv.load_dotenv()


class ModelConfig(BaseModel):
    model: str
    provider: str


class Provider(BaseModel):
    env_var: str
    base_url: str


class LLMConfigs(BaseSettings):
    models: typing.Dict[str, ModelConfig] = Field(default_factory=dict)
    providers: typing.Dict[str, Provider] = Field(default_factory=dict)

    model_config = SettingsConfigDict(
        env_nested_delimiter="__",
        env_prefix="LLM_CONFIGS__",
        nested_model_default_partial_update=True,
    )


class LLM:
    model_configs = LLMConfigs.model_validate({})

    def __init__(self):
        self.clients = {}

    def get_client(self, model) -> OpenAI:
        model_config = self.model_configs.models[model]
        provider = model_config.provider
        if provider not in self.clients:
            self.clients[provider] = self.create_client(provider)
        return self.clients[provider]

    def get_model(self, model_name):
        model_config = self.model_configs.models[model_name]
        return model_config.model

    @classmethod
    def create_client(cls, provider):
        if provider not in cls.model_configs.providers:
            raise ValueError(f"Unknown provider: {provider}")
        provider_config = cls.model_configs.providers[provider]
        api_key = os.getenv(provider_config.env_var)
        if not api_key:
            raise ValueError(f"No API key found for provider: {provider}")
        base_url = provider_config.base_url
        return OpenAI(
            api_key=api_key,
            base_url=base_url,
        )

    def chat_completion(
        self, model, text
    ) -> typing.Tuple[str, openai.types.Completion]:
        import time

        start = time.time()
        completion = self.get_client(model).chat.completions.create(
            model=self.get_model(model),
            messages=[{"role": "user", "content": text}],
        )
        elapsed = time.time() - start
        return completion.choices[0].message.content, completion, elapsed

    def chat(self, model, messages) -> typing.Tuple[openai.types.Completion, int]:
        import time

        start = time.time()
        completion = self.get_client(model).chat.completions.create(
            model=self.get_model(model),
            messages=messages,
        )
        elapsed = time.time() - start
        return completion, elapsed
