import logging
import os
from typing import Any, Callable, Dict, List, Tuple

from pydantic import BaseModel, Field

from .llm import LLM

logger = logging.getLogger(__name__)


class StatefulTranslator(BaseModel):
    translation_model: str = "deepseek-chat"
    score_model: str = "deepseek-chat"
    translation_usage: dict = Field(
        default_factory=lambda: {
            "completion_tokens": 0,
            "prompt_tokens": 0,
            "total_tokens": 0,
        }
    )
    score_usage: dict = Field(
        default_factory=lambda: {
            "completion_tokens": 0,
            "prompt_tokens": 0,
            "total_tokens": 0,
        }
    )
    total_words: int = 0
    translation_elapsed: float = 0
    score_elapsed: float = 0
    total_revise_count: int = 0
    total_translate_count: int = 0
    cache: dict = Field(default_factory=dict)

    def get_df(self, revise_index: int = 100):
        import pandas as pd

        data = self.get_ith_revise_output(revise_index)

        return pd.DataFrame(data)

    def get_ith_revise_output(self, revise_index: int):
        data = []
        for key, value in self.cache.items():
            record = {}
            score_history = value["score_history"]
            score_reason_history = value["score_reason_history"]
            messages = value["messages"]
            revise_count = value["revise_count"]

            index = min(revise_index, revise_count)
            score = score_history[index]
            score_reason = score_reason_history[index]
            translation = messages[(2 * index) + 1]["content"]

            record["ID"] = key
            record["Raw Text"] = value["raw_text"]
            record["Translated Text"] = translation
            record["Accuracy"] = score["Accuracy"]
            record["Fluency"] = score["Fluency"]
            record["Completeness"] = score["Completeness"]
            record["Style & Word Choice"] = score["Style & Word Choice"]
            record["Reason"] = score_reason
            record["Score History"] = score_history
            record["Score Reason History"] = score_reason_history
            record["Revise Count"] = revise_count
            record["Messages"] = messages
            record["Total Score"] = sum(score.values())
            data.append(record)
        return data

    def update_translation_usage(self, completion):
        if completion.usage is None:
            logger.warning(f"No usage for {self.translation_model} with {completion}")
            return
        self.translation_usage["completion_tokens"] += (
            completion.usage.completion_tokens
        )
        self.translation_usage["prompt_tokens"] += completion.usage.prompt_tokens
        self.translation_usage["total_tokens"] += completion.usage.total_tokens

    def update_score_usage(self, completion):
        if completion.usage is None:
            logger.warning(f"No usage for {self.score_model} with {completion}")
            return
        self.score_usage["completion_tokens"] += completion.usage.completion_tokens
        self.score_usage["prompt_tokens"] += completion.usage.prompt_tokens
        self.score_usage["total_tokens"] += completion.usage.total_tokens

    def update_translation_elapse(self, elapsed):
        self.translation_elapsed += elapsed

    def update_score_elapse(self, elapsed):
        self.score_elapsed += elapsed

    @property
    def llm_client(self):
        if not hasattr(self, "_llm_client"):
            self._llm_client = LLM()
        return self._llm_client

    def _translate(self, id: str, text: str) -> Tuple[str, list]:
        prompt = f"""请将以下的应标标书内容翻译英文：
1.	要求保持专业性、正式、严谨的语气。
2.	确保术语准确翻译，尤其是技术、法律或商务相关内容。
3.	保持原文结构和段落清晰度，适当调整语序以符合英语表达习惯。
4.  数字、单位和表格格式应保持一致，不要改变原文中的信息。
5.  请直接给出翻译，不要附带任何额外的解释。
6.  即使翻译内容很短，也请按照要求翻译，不要解释！
待翻译内容：
{{text}}
"""
        self.total_translate_count += 1
        request_text = prompt.format(text=text)
        messages = [{"role": "user", "content": request_text}]
        completion, elapsed = self.llm_client.chat(self.translation_model, messages)
        if len(completion.choices) == 0:
            raise ValueError(
                f"No completion for {self.translation_model} with {request_text}, {completion}"
            )
        messages.append(
            {"role": "assistant", "content": completion.choices[0].message.content}
        )
        self.update_translation_usage(completion)
        self.update_translation_elapse(elapsed)
        if logger.isEnabledFor(logging.DEBUG):
            logger.debug(
                f"({id})\n原文->{text}\n翻译->{completion.choices[0].message.content}"
            )
        return completion.choices[0].message.content, messages

    def _score_translation(self, id: str, text: str, translation: str) -> float:
        prompt = """你现在是一位专业的翻译质量评估专家。
我将提供一段中文原文，以及对应的英文译文。请你在阅读后，依据以下四个主要维度来对翻译进行评分:
1.	准确性（Accuracy）：译文是否正确传达了原文的含义，是否存在歪曲或错译等情况；
2.	流畅度（Fluency）：译文在英语层面的语法、表达是否通顺自然，是否符合英语使用习惯；
3.	完整性（Completeness）：译文中是否缺少原文的重要信息，是否有明显的增译或删减；
4.	风格与用词（Style & Word Choice）：译文的用词、措辞及整体风格是否符合原文意图，是否与语境、专业领域或目标读者相匹配。

具体评分标准参考：
•	5 分：几乎无可挑剔，准确传达，流畅自然，风格与原文高度契合；
•	4 分：基本准确、流畅，只有少量不影响整体意义的小问题；
•	3 分：存在一些明显的问题或不够通顺的地方，但大体可理解；
•	2 分：多处含义偏差或语法失误，影响阅读和理解；
•	1 分：严重偏离原文，错漏严重，几乎无法反映原文主旨。

请按照以下格式回复：
1.	准确性：X分
2.	流畅度：X分
3.	完整性：X分
4.	风格与用词：X分
建议：（可选）

若某个维度的评分不为5分，请详细说明原因，并提出能够改进到5分的建议。

以下是待评估的文本：
•	中文原文：{zh}
•	英文译文：{en}
"""
        messages = [{"role": "user", "content": prompt.format(zh=text, en=translation)}]
        completion, elapsed = self.llm_client.chat(self.score_model, messages)
        self.update_score_usage(completion)
        self.update_score_elapse(elapsed)

        import re

        def extract_ranking(ranking_text: str) -> dict:
            pattern = r"(\d+).*(准确性|流畅度|完整性|风格与用词).*?(\d)\s*分"
            scores = re.findall(pattern, ranking_text)
            if len(scores) == 8:
                scores = scores[:4]
            elif len(scores) != 4:
                raise ValueError(scores, ranking_text)

            scores = {
                "Accuracy": int(scores[0][2]),
                "Fluency": int(scores[1][2]),
                "Completeness": int(scores[2][2]),
                "Style & Word Choice": int(scores[3][2]),
            }
            return scores

        score_text = completion.choices[0].message.content
        return extract_ranking(score_text), score_text

    def _revice_translation(
        self, id: str, text: str, score_reason: str, messages
    ) -> Tuple[str, list]:
        import copy

        self.total_revise_count += 1
        messages = copy.deepcopy(messages)
        messages.append(
            {
                "role": "user",
                "content": f"我给你一份其他模型对你这份翻译的评价，你可以选择认同或者部分认同并基于此优化你的翻译，或者你可以坚持你的翻译。请直接输出翻译结果\n{score_reason}",
            }
        )
        completion, elapsed = self.llm_client.chat(
            self.translation_model,
            messages=messages,
        )
        messages.append(
            {"role": "assistant", "content": completion.choices[0].message.content}
        )
        self.update_translation_usage(completion)
        self.update_translation_elapse(elapsed)
        return completion.choices[0].message.content, messages

    def _translate_with_revise(
        self,
        id: str,
        text: str,
        stop_criteria: Callable[[Dict[str, Any]], bool],
        max_revise_count: int = 10,
        not_improved_count: int = 5,
    ) -> Tuple[str, float, str, list, int, List[Dict[str, float]], List[str]]:
        score_history = []
        score_reason_history = []
        translation, messages = self._translate(id, text)
        score, score_reason = self._score_translation(id, text, translation)
        score_history.append(score)
        score_reason_history.append(score_reason)
        revise_count = 0
        for _ in range(max_revise_count):
            if stop_criteria(id, score, score_history, not_improved_count):
                return (
                    translation,
                    score,
                    score_reason,
                    messages,
                    revise_count,
                    score_history,
                    score_reason_history,
                )

            revise_count += 1
            logger.info(
                f"({id})Revising translation({revise_count}), score_history: {[sum(score.values()) for score in score_history]}"
            )
            translation, messages = self._revice_translation(
                id, text, score_reason, messages
            )
            score, score_reason = self._score_translation(id, text, translation)
            score_history.append(score)
            score_reason_history.append(score_reason)

        logger.info(
            f"({id})Stop revising translation since max revise count is reached"
        )
        return (
            translation,
            score,
            score_reason,
            messages,
            revise_count,
            score_history,
            score_reason_history,
        )

    def stop_criteria(
        self,
        id: str,
        score: Dict[str, float],
        score_history: List[Dict[str, float]],
        not_improved_count: int = 2,
    ) -> bool:
        if sum(score.values()) == 20:
            logger.info(f"({id})Stop revising translation since score is 20")
            return True

        if len(score_history) <= not_improved_count:
            return False

        improved = []
        for i in range(1, not_improved_count + 1):
            improved.append(
                score_history[-i]["Accuracy"] > score_history[-i - 1]["Accuracy"]
                and score_history[-i]["Fluency"] > score_history[-i - 1]["Fluency"]
                and score_history[-i]["Completeness"]
                > score_history[-i - 1]["Completeness"]
                and score_history[-i]["Style & Word Choice"]
                > score_history[-i - 1]["Style & Word Choice"]
            )

        if any(improved):
            return False

        logger.info(
            f"({id})Stop revising translation since score is not improved in {not_improved_count} times"
        )
        return True

        # return (
        #     score["Accuracy"] >= 4
        #     and score["Fluency"] >= 4
        #     and score["Completeness"] >= 4
        #     and score["Style & Word Choice"] >= 4
        #     and sum(score.values()) >= 17
        # )

    def translate(self, id: str, text: str, label: str) -> str:
        if id not in self.cache:
            (
                response_text,
                score,
                reason,
                messages,
                revise_count,
                score_history,
                score_reason_history,
            ) = self._translate_with_revise(
                id, text, self.stop_criteria, max_revise_count=10, not_improved_count=5
            )
            self.total_words += len(text.strip())
            self.cache[id] = {
                "raw_text": text,
                "text": response_text,
                "score": score,
                "score_history": score_history,
                "score_reason_history": score_reason_history,
                "reason": reason,
                "messages": messages,
                "revise_count": revise_count,
                "label": label,
            }
        return self.cache[id]["text"]

    def save(self, path: str = "stateful_translator.json"):
        with open(path, "w") as f:
            f.write(self.model_dump_json())

    @classmethod
    def load(
        cls,
        path: str = "stateful_translator.json",
        translation_model: str = "deepseek-chat",
        score_model: str = "deepseek-chat",
    ):
        if os.path.exists(path):
            return cls.model_validate_json(open(path).read())
        return cls(translation_model=translation_model, score_model=score_model)
